import dash_html_components as html
import dash_core_components as dcc
import dash_table
import pandas as pd
import numpy as np
import bifrost_dashboard.components.import_data as import_data
import dash_bootstrap_components as dbc
import bifrost_dashboard.components.global_vars as global_vars

TABLE_PAGESIZE = 100


def format_selected_samples(filtered_df):
    "Returns a formatted string of selected samples"
    return "\n".join([row["name"] for index, row in filtered_df.iterrows()])


def html_collection_selector():
    return html.Div(
        [
            html.Div(
                [
                    html.Div(
                        [
                            html.Div(
                                dbc.Label(
                                    "Select Collection",
                                    html_for="collection-selector",
                                    className="m-0 font-weight-bold text-primary h6 d-block"),
                                className="card-header py-3"
                            ),
                            html.Div(
                                [
                                    dbc.Row([
                                        dbc.Col([
                                            dbc.FormGroup(
                                                [
                                                    dcc.Dropdown(
                                                        id="collection-selector",
                                                        value=None,
                                                    ),
                                                ]
                                            ),
                                        ], lg=9),
                                        dbc.Col([
                                            dbc.FormGroup(
                                                [
                                                    dcc.Link(
                                                        "Load collection",
                                                        id="collection-link",
                                                        href="/collection",
                                                        className="btn btn-primary"),
                                                ]
                                            )
                                        ], lg=3),
                                    ])
                                ],
                                className="card-body"
                            )
                        ], className="card shadow mb-4"
                    )
                ],
                className="col-lg-9"
            )

            # html.Div(id="tsv-download")
        ],
        className="row", id="collection-selector-div"
    )


# callback
def update_collection_button(collection, pathname):
    if pathname is None or pathname == "/":
        pathname = "/"
    path = pathname.split("/")
    if path[1] == "resequence-report":
        if collection is not None:
            return "/resequence-report/" + collection
        else:
            return "/resequence-report"
    else:
        if collection is not None:
            return "/collection/" + collection
        else:
            return "/collection"


def html_filter_drawer():
    qc_list_options = [
        {"label": "OK", "value": "OK"},
        {"label": "Core Facility", "value": "core facility"},
        {"label": "Supplying Lab", "value": "supplying lab"},
        {"label": "Not checked", "value": "Not checked"}
    ]
    run_filter = html.Div(
        [
            html.Div(
                [
                    html.Div(
                        [
                            html.Div(
                                html.H6(
                                    "Filter",
                                    className="m-0 font-weight-bold text-primary"
                                ),
                                className="card-header py-3"
                            ),
                            html.Div(
                                [
                                    dbc.Row([
                                        dbc.Col([
                                            dbc.Row(
                                                [
                                                    dbc.Col(
                                                        dbc.FormGroup(
                                                            [
                                                                dbc.Label(
                                                                    "Run", html_for="run-list"),
                                                                dcc.RadioItems(
                                                                    options=[
                                                                        {"label": " Routine",
                                                                         "value": "routine"},
                                                                        {"label": "Custom",
                                                                         "value": "custom"},
                                                                    ],
                                                                    value="routine",
                                                                    labelStyle={
                                                                        'margin': '0 0 0.5rem 0.5rem'},
                                                                    id="form-run-show-custom",
                                                                    style={
                                                                        "display": "inline-block"}

                                                                ),
                                                                dcc.Dropdown(
                                                                    id="run-list",
                                                                    multi=True,
                                                                    value=[],
                                                                    placeholder="All runs selected",
                                                                ),
                                                            ]
                                                        ),
                                                        width=12, id="run-list-div"
                                                    ),
                                                    dbc.Col(
                                                        dbc.FormGroup(
                                                            [
                                                                dbc.Label(
                                                                    "Group", html_for="group-list"),
                                                                dcc.Dropdown(
                                                                    id="group-list",
                                                                    multi=True,
                                                                    value=[],
                                                                    placeholder="All groups selected",
                                                                ),
                                                            ]
                                                        ),
                                                        width=6,
                                                    ),
                                                    dbc.Col(
                                                        dbc.FormGroup(
                                                            [
                                                                html.Label(
                                                                    [
                                                                        "Species: "
                                                                    ],
                                                                    htmlFor="species-list",
                                                                    style={
                                                                        "display": "inline-block"}
                                                                ),
                                                                dcc.RadioItems(
                                                                    options=[
                                                                        {"label": " Provided",
                                                                         "value": "provided"},
                                                                        {"label": " Detected",
                                                                         "value": "detected"},
                                                                    ],
                                                                    value="provided",
                                                                    labelStyle={
                                                                        'margin': '0 0 0.5rem 0.5rem'},
                                                                    id="form-species-source",
                                                                    style={
                                                                        "display": "inline-block"}

                                                                ),
                                                                html.Div(
                                                                    dcc.Dropdown(
                                                                        id="species-list",
                                                                        multi=True,
                                                                        value=[],
                                                                        placeholder="All species selected",
                                                                    ),
                                                                    id="species-div"
                                                                )
                                                            ]
                                                        ),
                                                        width=6,
                                                    ),
                                                    dbc.Col(
                                                        dbc.FormGroup(
                                                            [
                                                                dbc.Label(
                                                                    "SSI QC", html_for="qc-list"),
                                                                dcc.Dropdown(
                                                                    id="qc-list",
                                                                    multi=True,
                                                                    options=qc_list_options,
                                                                    placeholder="All values selected",
                                                                    value=[]
                                                                ),
                                                            ]
                                                        ),
                                                        width=6,
                                                    ),
                                                    dbc.Col(
                                                        dbc.FormGroup(
                                                            [
                                                                dbc.Label(
                                                                    "Date sequenced", html_for="date-sequenced"),
                                                                html.Div(
                                                                    dcc.DatePickerRange(
                                                                        id='date-sequenced'
                                                                    ),
                                                                )
                                                            ]
                                                        ),
                                                        width=6,
                                                    ),
                                                ],
                                                form=True,
                                            ),
                                        ], width=9),
                                        dbc.Col([
                                            dbc.Row(dbc.Col([
                                                dbc.Label(["Sample names ", html.Span(
                                                    "(?)", id="sample-names-tooltip")],
                                                    html_for="samples-form",
                                                    style={"display": "block"}),
                                                dbc.Textarea(
                                                    id="samples-form",
                                                    placeholder="one sample per line",
                                                    value="",
                                                    rows=6
                                                ),
                                                dbc.Tooltip(
                                                    "Sample name must match exactly. "
                                                    "Search samples using regex by starting and "
                                                    "finishing sample name with the '/' character.",
                                                    target="sample-names-tooltip",
                                                )
                                            ], width=12))
                                        ], width=3),
                                    ]),
                                    dbc.ButtonGroup([
                                        dbc.Button("Search samples",
                                                   id="apply-filter-button",
                                                   color="primary",
                                                   n_clicks=0),
                                        # dbc.Button("Generate download link (1000 samples max.)",
                                        #            id="generate-download-button",
                                        #            color="secondary",
                                        #            n_clicks=0)
                                    ]),
                                ],
                                className="card-body"
                            )
                        ], className="card shadow mb-4"
                    )
                ],
                className="col"
            )
            
            # html.Div(id="tsv-download")
        ],
        className="row"
    )

    return run_filter


def html_div_filter():
    return html.Div([
        html.Div(
            [
                html.Div([
                    html.Div(
                        [
                            html.H6("List view",
                                    className="m-0 font-weight-bold text-primary"),
                        ],
                        className="card-header py-3"
                    ),

                    html.Div([
                        html.Div([
                            html.Div([
                                html.Div([
                                    dbc.Button("Add to selection",
                                               id="add-selection-button",
                                               n_clicks=0,
                                               n_clicks_timestamp=-1,
                                               size="sm",
                                               disabled=False)
                                ]),
                            ], className="col-auto mr-auto"),
                            html.Div([

                                html.Span(
                                    "0", id="filter-sample-count"),
                                ' samples loaded.',

                                dbc.ButtonGroup([
                                    dbc.Button(html.I(className="fas fa-download fa-sm fa-fw"),
                                               id="generate-download-button",
                                               color="secondary",
                                               size="sm",
                                               className="ml-2",
                                               n_clicks=0)
                                ]),
                                html.Div(id="tsv-download")
                            ], className="col-auto"),
                        ], className="row mb-3"),
                        html.Div([], id="placeholder0"),
                        dash_table.DataTable(
                            data=[{}],
                            style_table={
                                'overflowX': 'scroll',
                            },
                            columns=global_vars.COLUMNS,
                            style_cell={
                                'minWidth': '200px',
                                'textAlign': 'center',
                                "fontFamily": "Arial",
                                "padding": "0px 10px",
                                "fontSize": "0.7rem",
                                "height": "auto"
                            },
                            style_cell_conditional=[
                                {
                                    "if": {"column_id": "ssi_stamper_failed_tests"},
                                    "textAlign": "left"
                                }
                            ],
                            fixed_rows={'headers': True},
                            row_selectable='multi',
                            # filtering=True,  # Front end filtering
                            # sorting=True,
                            selected_rows=[],
                            # style_data_conditional=style_data_conditional,
                            # pagination_settings={
                            #     'current_page': 0,
                            #     'page_size': TABLE_PAGESIZE
                            # },
                            virtualization=False,
                            page_action='none',
                            id="datatable-ssi_stamper")
                    ], className="card-body bigtable")

                ], id="ssi_stamper-report", className="card shadow mb-4"),
            ]
        )
    ])


def generate_table(tests_df):
    qc_action = "properties.stamper.summary.stamp.value"
    user_stamp_col = "properties.stamper.summary.stamp.name"
    r1_col = "properties.datafiles.summary.paired_reads"

    # Add needed columns
    for col in [qc_action, user_stamp_col, r1_col]:
        if col not in tests_df:
            tests_df[col] = np.nan

    # Convert to string for comparison later on
    tests_df = tests_df.astype({user_stamp_col: str})

    mask = pd.isnull(tests_df[qc_action])
    tests_df.loc[mask, qc_action] = "not tested"
    slmask = tests_df[qc_action] == "supplying lab"
    tests_df.loc[slmask, qc_action] = "warning: supplying lab"

    user_mask = tests_df[user_stamp_col] == "user_feedback"
    tests_df.loc[user_mask, qc_action] = "👤 " + tests_df.loc[user_mask, qc_action]

    test_cols = [col.split(".")[-2] for col in tests_df.columns
                 if (col.startswith("properties.stamper.summary.") and
                 col != "properties.stamper.summary.stamp.status" and
                 col.endswith(".status"))]

    # Round columns:
    for col in global_vars.ROUND_COLUMNS:
        if col in tests_df.columns:
            tests_df[col] = round(tests_df[col], 3)

    def concatenate_failed(row):
        res = []
        tests = {}
        for test_name in test_cols:
            reason_c = "properties.stamper.summary.{}.reason".format(test_name)
            value_c = "properties.stamper.summary.{}.value".format(test_name)
            status_c = "properties.stamper.summary.{}.status".format(test_name)
            if pd.isna(row[value_c]):
                tests[test_name] = {
                    "status": "fail",
                    "reason": "Not tested",
                    "value": ""
                }
            else:
                status = row[status_c]
                reason = row[reason_c]
                value = row[value_c]
            
                tests[test_name] = {
                    "status": status,
                    "reason": reason,
                    "value": value
                }
                
        for test_name in tests:
            test = tests[test_name]
            if test["status"] == "fail":
                res.append("Test {}: {}, {}".format(
                    test_name, test["status"], test["reason"]))
        row["ssi_stamper_failed_tests"] = ". ".join(res)
        return row

    tests_df = tests_df.apply(concatenate_failed, axis="columns")

    COLUMNS = global_vars.COLUMNS

    # Generate conditional formatting:
    style_data_conditional = []
    conditional_columns = ["properties.stamper.summary.stamp.value"]

    for status, color in ("fail", "#ea6153"), ("undefined", "#f1c40f"):
        style_data_conditional += list(map(lambda x, s=status, c=color: {"if": {
            "column_id": x, "filter": '{} eq "{}"'.format(x, s)}, "backgroundColor": c}, conditional_columns))

    for status, color in ("core facility", "#ea6153"), ("warning: supplying lab", "#f1c40f"):
        style_data_conditional += [{"if": {
            "column_id": qc_action, "filter": 'QC_action eq "{}"'.format(status)}, "backgroundColor": color}]

    tests_df = tests_df.rename({"_id": "id"}, axis="columns")
    tests_df["id"] = tests_df["id"].astype(str)

    tests_df = tests_df.filter([c["id"] for c in COLUMNS])

    return tests_df


# callback
def filter_update_run_options(form_species, selected_collection, run_type):
    # Runs
    if run_type == "custom":
        run_type = ["project", "external", "virtual"]
    else:
        run_type = "routine"
    run_list = import_data.get_run_list(run_type)
    run_options = [
        {
            "label": "{} ({})".format(run["name"],
                                      len(run["samples"])),
            "value": run["name"]
        } for run in run_list]

    # Groups
    group_list = import_data.get_group_list(selected_collection)
    group_options = []
    for item in group_list:
        if pd.isnull(item["_id"]):
            group_options.append({
                "label": "Not defined ({})".format(item["count"]),
                "value": "Not defined"
            })
        else:
            group_options.append({
                "label": "{} ({})".format(item["_id"], item["count"]),
                "value": item["_id"]
            })

    species_list = import_data.get_species_list(
        form_species, selected_collection)

    species_options = []
    for item in species_list:
        if pd.isna(item["_id"]):
            species_options.append({
                "label": "Not classified",
                "value": "Not classified"
            })
        else:
            species_options.append({
                "label": item["_id"],
                "value": item["_id"]
            })
    return [run_options, run_options, group_options, species_options]


# callback
def filter_update_filter_values(param_store):
    runs = param_store.get("run", [])
    groups = param_store.get("group", [])
    species = param_store.get("species", [])
    qcs = param_store.get("qc", [])
    sample_names = param_store.get("sample_names", [])
    return [runs, groups, species, qcs, "\n".join(sample_names)]



# -*- coding: utf-8 -*-
import os
import re
import urllib.parse as urlparse
import datetime
from flask_caching import Cache

import dash
import dash_core_components as dcc
import dash_html_components as html
import dash_auth
import numpy as np
import dash_bootstrap_components as dbc
from dash.dependencies import ClientsideFunction, Input, Output, State
import dash_table


import bifrostapi

import bifrost_dashboard.components.import_data as import_data
from bifrost_dashboard.components.filter import html_div_filter, generate_table, filter_update_run_options, filter_update_filter_values, html_filter_drawer, html_collection_selector, update_collection_button
from bifrost_dashboard.components.sample_report import SAMPLE_PAGESIZE, sample_report, children_sample_list_report, samples_next_page
import bifrost_dashboard.components.global_vars as global_vars
import bifrost_dashboard.components.admin as admin
from bifrost_dashboard.run_checker import pipeline_report, rerun_components_button, update_rerun_table, pipeline_report_data
from bifrost_dashboard.components.aggregate_report import aggregate_report, update_aggregate_fig, aggregate_species_dropdown
from bifrost_dashboard.components.resequence_report import resequence_report
from bifrost_dashboard.components.link_to_files import link_to_files, link_to_files_div
import bifrost_dashboard.components.virtual_runs as virtual_runs

import yaml
config = yaml.safe_load(open(os.environ["BIFROST_DASH_CONFIG"]))


bifrostapi.add_URI(config["mongodb_key"])

external_scripts = [
    'https://kit.fontawesome.com/24170a81ff.js',
]

external_stylesheets = [
    "https://fonts.googleapis.com/css?family=Lato",
    dbc.themes.BOOTSTRAP
]
assets = os.path.dirname(os.path.abspath(__file__)) + "/data/assets"
app = dash.Dash("bifrost_dashboard",
    assets_folder=assets,
    external_stylesheets=external_stylesheets,
    external_scripts=external_scripts
)
app.title = "bifrost"
app.config["suppress_callback_exceptions"] = True
cache = Cache(app.server, config={
    'CACHE_TYPE': 'filesystem',
    'CACHE_DIR': config["cache_location"]
})
cache_timeout = 60

if config.get("pass_protected"):
    dash_auth.BasicAuth(
        app,
        config.USERNAME_PASSWORD
    )

# Temp css to make it look nice
# Lato font


def samples_list(active, collection_name=None):
    links = [
        {
            "icon": "fa-list",
            "href": ""
        },
        {
            "icon": "fa-money-check",
            "href": "sample-report"
        },
        {
            "icon": "fa-chart-pie",
            "href": "aggregate"
        },
        {
            "icon": "fa-traffic-light",
            "href": "pipeline-report"
        },
        {
            "icon": "fa-link",
            "href": "link-to-files"
        }
    ]
    link_list = []
    for item in links:
        href = "/" + item["href"]
        if collection_name is not None:
            href = "/collection/{}/{}".format(collection_name, item["href"])
        if active == item['href']:
            link_list.append(dcc.Link(
                html.I(className="fas {} fa-fw".format(item['icon'])),
                className="btn btn-outline-secondary active",
                href=href
            ))
        else:
            link_list.append(dcc.Link(
                html.I(className="fas {} fa-fw".format(item['icon'])),
                className="btn btn-outline-secondary",
                href=href
            ))
    return link_list

app.layout = html.Div([
    dcc.Location(id="url", refresh=False),
    # To store url param values
    dcc.Store(id="sample-store", data=[], storage_type='session'),
    dcc.Store(id="param-store", data={}),
    dcc.Store(id="selected-collection", data=None),
    html.Ul(
        [
            html.A(
                [
                    html.Img(src="/assets/img/bifrost-logo-white@2x.png",
                             className="navbar-logo ")
                    # html.Div("bifrost", className="sidebar-brand-text mx-3")
                ],
                className="sidebar-brand d-flex align-items-center justify-content-center",
                href="/"
            ),
            
            html.Hr(className="sidebar-divider"),
            html.Div("Browse data", className="sidebar-heading"),

            html.Li(dcc.Link(
                [
                    html.I(className="fas fa-vial fa-fw"),
                    html.Span("Samples")
                ], className="nav-link", href="/"),
                className="nav-item",
                id="samples-nav"),
            html.Li(dcc.Link([
                    html.I(className="fas fa-vials fa-fw"),
                    html.Span("Collections")
                    ], className="nav-link", href="/collection"),
                    className="nav-item", id="collections-nav"),
            html.Hr(className="sidebar-divider"),
            html.Div("Reports", className="sidebar-heading"),

            html.Li(dcc.Link(
                [
                    html.I(className="fas fa-chart-line fa-fw"),
                    html.Span("Resequence report")
                ], className="nav-link", href="/resequence-report"),
                className="nav-item", id="resequence-nav"),
            html.Hr(className="sidebar-divider"),
            html.Div(
                html.Button(className="rounded-circle border-0",
                            id="sidebarToggle"),
                className="text-center d-none d-md-inline"
            ),
        ],
        className="navbar-nav bg-gradient-primary sidebar sidebar-dark accordion",
        id="sidebar"
    ),
    html.Div([
        html.Div(id="content", children=[
            html.Nav(
                [
                    html.Ul([
                        html.Li(
                            html.Span("This view is in Beta. Please report any feedback/bugs to mbas@ssi.dk :)"), className="nav-item mx-1"
                        ),
                    ], className="navbar-nav"),
                    html.Ul([
                        html.Li(
                            html.A("Documentation", href="https://ssi-dk.github.io/bifrost/"), className="nav-item dropdown no-arrow mx-1"
                        ),
                        html.Li(
                            html.A("Github", href="https://github.com/ssi-dk/bifrost"), className="nav-item dropdown no-arrow mx-1"
                        )
                    ],className="navbar-nav ml-auto")
                ],
                className="navbar navbar-expand navbar-light bg-white topbar mb-4 static-top shadow"),
            html.Main([
                html_collection_selector(),
                html.Div([
                    dbc.Collapse(
                        [
                            html_filter_drawer()
                        ], id="filter_panel"
                    ),
                    html.Div([
                        html.Div([
                            html.Div(
                                samples_list('/'),
                                className="btn-group shadow-sm",
                                id="selected-view-buttons"
                            ),
                        ], className="col-4"),
                        html.Div([
                            html.Button(
                                html.I(className="fas fa-filter fa-sm"),
                                className="btn btn-outline-secondary shadow-sm mx-auto d-block",
                                id="filter_toggle"
                            ),
                        ], className="col-4"),
                        html.Div([
                            html.Button(
                                [
                                    html.I(className="fas fa-shopping-cart fa-sm mr-2"),
                                    html.Span(0, id="selection-count")
                                ],
                                className="btn btn-outline-secondary shadow-sm mx-auto d-block",
                                id="selection-modal-open"
                            ),
                        ], className="col-4"),
                    ], className="row mb-4"),
                ], id="samples-panel", className="d-none"),
                html.Div(id="selected-view"),
                virtual_runs.modal,
                html.Footer([
                    "Created with 🔬 at SSI. Bacteria icons from ",
                    html.A("Flaticon", href="https://www.flaticon.com/"),
                    "."], className="footer container")
            ], className="container-fluid",
            role="main"),
        ]),
    ], id="content-wrapper", className="d-flex flex-column")
], id="wrapper")


# Callbacks

# We could make this one much faster by hiding the unused species with CSS
# by adding a new hidden class.



@app.callback(
    [Output("filter_panel", "is_open"),
     Output("filter_toggle", "className")],
    [Input("filter_toggle", "n_clicks")],
    [State("filter_panel", "is_open")]
)
def sidebar_toggle(n_clicks, is_open):
    if n_clicks:
        if is_open:
            return [False, "btn btn-outline-secondary shadow-sm mx-auto d-block"]
        else:
            return [True, "btn btn-outline-secondary shadow-sm mx-auto d-block active"]
    return [is_open, "btn btn-outline-secondary shadow-sm mx-auto d-block"]


@app.callback(
    Output("param-store", "data"),
    [Input("url", "search")],
    [State("param-store", "data")]
)
def update_run_name(params, prev_params):
    if params is None or params == "":
        raise dash.exceptions.PreventUpdate("Initial repeated params call")
    pparse = urlparse.urlparse(params)
    params = urlparse.parse_qs(pparse.query)
    if params == prev_params:
        raise dash.exceptions.PreventUpdate("No param change")
    return params

@app.callback(
    [Output("selected-view", "children"),
     Output("selected-view-buttons", "children"),
     Output("samples-panel", "className"),
     Output("samples-nav", "className"),
     Output("resequence-nav", "className"),
     Output("collections-nav", "className"),
     Output("selected-collection", "data"),
     Output("collection-selector-div", "className"),
     Output("run-list-div", "className")],
    [Input("url", "pathname")],
    [State("sample-store", "data")]
)
def update_view(pathname, sample_store):

    if pathname is None or pathname == "/":
        pathname = "/"
    path = pathname.split("/")
    view = None
    samples_panel = ""
    samples_nav = "nav-item"
    resequence_nav = "nav-item"
    collections_nav = "nav-item"
    collection_view = False
    collection_name = None
    if path[1] == "collection":
        collection_view = True
        if len(path) > 2: #/collection/collectionname
            collection_name = path[2]
            if len(path) > 3:  # /collection/collectionname/section
                section = path[3]
            else:  # /collection/collectionname
                section = ""
        else:  # /collection
            section = ""
    else:  # /section
        section = path[1]
        if section == "resequence-report":
            if len(path) == 3:  #/resequence-report/collectionname
                collection_name = path[2]
            resequence_nav += " active" 
        else:
            samples_nav += " active"


    if section == "":
        view = html_div_filter()
    elif section == "sample-report":
        view = sample_report(sample_store)
    elif section == "aggregate":
        view = aggregate_report()  # Doesn't need data
    elif section == "pipeline-report":
        view = pipeline_report(sample_store)
    elif section == "resequence-report":
        samples_panel = "d-none"
        view = resequence_report(collection_name)
    elif section == "link-to-files":
        view = link_to_files_div()
    else:
        samples_panel = "d-none"
        view = "Not found"
    
    if collection_view:
        collection_selector_list = "row"
        run_list = "d-none"
        collections_nav += " active"
    elif section == "resequence-report":
        collection_selector_list = "row"
        run_list = "d-none"
    else:
        collection_selector_list = "row d-none"
        run_list = ""

    return [view, samples_list(section, collection_name), samples_panel,
            samples_nav, resequence_nav, collections_nav, collection_name,
            collection_selector_list, run_list]

@app.callback(
    [Output("run-list", "options"), 
     Output("collection-selector", "options"),
     Output("group-list", "options"),
     Output("species-list", "options")],
    [Input("form-species-source", "value"),
    Input("selected-collection", "data"),
    Input("form-run-show-custom", "value")
    ]
)
@cache.memoize(timeout=cache_timeout)  # in seconds
def update_run_options(form_species, selected_collection, run_type):
    return filter_update_run_options(form_species, selected_collection, run_type)


@app.callback(
    Output("collection-selector", "value"),
    [Input("selected-collection", "data")]
)
def update_selected_collection(selected_collection):
    return selected_collection

@app.callback(
    [Output("run-list", "value"),
     Output("group-list", "value"),
     Output("species-list", "value"),
     Output("qc-list", "value"),
     Output("samples-form", "value")],
    [Input("param-store", "data")]
)
def update_filter_values(param_store):
    return filter_update_filter_values(param_store)


@app.callback(
    Output("collection-link", "href"),
    [Input("collection-selector", "value")],
    [State("url", "pathname")]
)
def update_collection_button_f(collection, pathname):
    return update_collection_button(collection, pathname)

@app.callback(
    Output("page-n",
            "children"),
    [Input("prevpage", "n_clicks_timestamp"),
        Input("prevpage2", "n_clicks_timestamp"),
        Input("nextpage", "n_clicks_timestamp"),
        Input("nextpage2", "n_clicks_timestamp")],
    [State("page-n", "children"),
        State("max-page", "children")]
)
def next_page(prev_ts, prev_ts2, next_ts, next_ts2, page_n, max_page):
    return samples_next_page(prev_ts, prev_ts2, next_ts, next_ts2, page_n, max_page)


@app.callback(
    Output("sample-report", "children"),
    [Input("page-n", "children"),
    Input("sample-store", "data")]
        )
def fill_sample_report(page_n, sample_store):
    page_n = int(page_n)
    sample_ids = list(
        map(lambda x: x["_id"], sample_store))
    if len(sample_ids) == 0:
        return None
    
    data_table = import_data.filter_all(
        sample_ids=sample_ids,
        pagination={"page_size": SAMPLE_PAGESIZE, "current_page": page_n})
    max_page = len(sample_store) // SAMPLE_PAGESIZE
    # We need to have fake radio buttons with the same ids to account for times 
    # when not all SAMPLE_PAGESIZE samples are shown and are not taking the ids required by the callback
    html_fake_radio_buttons = html.Div([dcc.RadioItems(
        options=[
            {'label': '', 'value': 'nosample'}
        ],
        value='noaction',
        id="sample-radio-{}".format(n_sample)
    ) for n_sample in range(len(data_table), SAMPLE_PAGESIZE)], style={"display": "none"})
    return [
        html.H4("Page {} of {}".format(page_n + 1, max_page + 1)),
        html.Div(children_sample_list_report(data_table)),
        html_fake_radio_buttons,
        admin.html_qc_expert_form(),
        html.H4("Page {} of {}".format(page_n + 1, max_page + 1)),
        dcc.ConfirmDialog(
            id='qc-confirm',
            message='Are you sure you want to send sample feedback?',
        )
    ]


@app.callback(
    Output("sample-store", "data"),
    [Input("apply-filter-button", "n_clicks"),
     Input("param-store", "data"),
     Input("selected-collection", "data")],
    [State("run-list", "value"),
     State("species-list", "value"),
     State("form-species-source", "value"),
     State("group-list", "value"),
     State("qc-list", "value"),
     State("samples-form", "value"),
     State("sample-store", "data"),
     State("date-sequenced", "start_date"),
     State("date-sequenced", "end_date"),
     ]
)
def update_selected_samples(n_clicks, param_store, collection_name,
                            run_names, species_list,
                            species_source, group_list, qc_list,
                            sample_names, prev_sample_store,
                            date_seq_start, date_seq_end):
    date_range = [date_seq_start, date_seq_end]
    for i in range(2):
        if date_range[i] is not None:
            date_range[i] = datetime.datetime.strptime(re.split('T| ', date_range[i])[0], '%Y-%m-%d')

    if sample_names is not None and sample_names != "":
        sample_names = sample_names.split("\n")
    else:
        sample_names = param_store.get("sample_names", [])
    if not run_names:
        run_names = param_store.get("run", [])
    if not group_list:
        group_list = param_store.get("group", [])
    if not species_list:
        species_list = param_store.get("species", [])
    if not qc_list:
        qc_list = param_store.get("qc", [])
    if not date_range[0]:
        date_range[0] = param_store.get("date_seq_start", None)
    if not date_range[1]:
        date_range[1] = param_store.get("date_seq_end", None)

    #override if selected collection
    if collection_name is not None:
        run_names = [collection_name]

    if (date_range[0] is None and
            date_range[1] is None):
        date_range = None

    if (n_clicks == 0 and
        sample_names == [] and
        run_names == [] and
        group_list == [] and
        species_list == [] and
        qc_list == [] and
        date_range is None):
        samples = prev_sample_store
    else:
        

        samples = import_data.filter_all(
            species=species_list, species_source=species_source,
            group=group_list, qc_list=qc_list,
            run_names=run_names,
            sample_names=sample_names,
            date_range=date_range,
            projection={"name": 1})

        if "_id" in samples:
            samples["_id"] = samples["_id"].astype(str)
        samples = samples.to_dict('records')
    # if deleted_samples:
    #     samples = [s for s in samples if s["_id"] not in deleted_samples]
    return samples


@app.callback(
    [
        Output("filter-sample-count", "children"),
        Output("datatable-ssi_stamper", "data"),
        Output("datatable-ssi_stamper", "virtualization")
    ],
    [
     Input("placeholder0", "children"),
     Input("sample-store", "data")
    ],
)
def update_filter_table(_, sample_store):
    if len(sample_store) == 0:
        return ["0", [{}], False]
    sample_ids = list(
        map(lambda x: x["_id"], sample_store))

    samples = import_data.filter_all(
        sample_ids=sample_ids)

    samples = generate_table(samples)
    if len(sample_store) > 500:
        virtualization = True
    else:
        virtualization = False
    return [len(sample_store), samples.to_dict("rows"), virtualization]

@app.callback(
    Output("tsv-download", "children"),
    [Input("generate-download-button", "n_clicks")],
    [State("run-list", "value"),
     State("species-list", "value"),
     State("form-species-source", "value"),
     State("group-list", "value"),
     State("qc-list", "value"),
     State("samples-form", "value")],
    prevent_initial_call=True
)
def generate_download_button(download_button,
                            run_names, species_list,
                            species_source, group_list, qc_list,
                            sample_names):
    if download_button == 0:
        return None
    else:
        if sample_names is not None and sample_names != "":
            sample_names = sample_names.split("\n")

        tests_df = import_data.filter_all(species=species_list, species_source=species_source,
                                                       group=group_list, qc_list=qc_list,
                                                       run_names=run_names,
                                                       sample_names=sample_names,
                                                       pagination=None)
    # return samples.to_dict()
    if not len(tests_df):
        return None

    tests_df = generate_table(tests_df)

    rename_dict = {item["id"]: item["name"]
        for item in global_vars.COLUMNS}

    renamed = tests_df.rename(rename_dict, axis='columns')

    missing_columns = [a for a in list(
        rename_dict.values()) if not a in list(renamed.columns)]

    # add missing columns
    for column in missing_columns:
        renamed[column] = np.nan

    # reorder columns
    renamed = renamed[list(rename_dict.values())]

    csv_string_eur = renamed.to_csv(
        index=False, encoding="utf-8", sep=";", decimal=",")
    tsv_string_us = renamed.to_csv(index=False, encoding="utf-8", sep="\t")
    full_csv_string_eur = 'data:text/csv;charset=utf-8,' + \
        urlparse.quote(csv_string_eur)
    full_tsv_string_us = 'data:text/tab-separated-values;charset=utf-8,' + \
        urlparse.quote(tsv_string_us)
    return [
        html.A("(tsv, US format)",
                href=full_tsv_string_us,
                download='report.tsv'),
        " - ",
        html.A("(csv, EUR Excel format)",
                href=full_csv_string_eur,
                download='report.csv')
    ]


@app.callback(
    [Output("plot-species", "value"),
     Output("plot-species", "options")],
    [Input("sample-store", "data"),
     Input("plot-species-source", "value")],
    [State("plot-species", "value")]
)
def aggregate_species_dropdown_f(sample_store, plot_species, selected_species):
    return aggregate_species_dropdown(sample_store, plot_species, selected_species)


@app.callback(
    [Output("pipeline-table", "data"),
     Output("pipeline-table", "columns"),
     Output("pipeline-table", "style_data_conditional"),
     Output("rerun-samples", "options"),
     Output("rerun-components", "options")],
    [Input("sample-store", "data"),
     Input("table-interval", "n_intervals")]
)
def pipeline_report_data_f(sample_store, _):
    return pipeline_report_data(sample_store)


@app.callback(
    [Output("summary-plot", "figure"),
     Output("mlst-plot", "figure")],
    [Input("plot-species", "value")],
    [State("sample-store", "data"),
    State("plot-species-source", "value")]
)
@cache.memoize(timeout=cache_timeout)  # in seconds
def update_aggregate_fig_f(selected_species, samples, plot_species_source):
    return update_aggregate_fig(selected_species, samples, plot_species_source)

@app.callback(Output("pipeline-rerun", "data"),
              [Input("pipeline-table", "active_cell"),
               Input("pipeline-table", "derived_viewport_data"),
               Input("rerun-add-components", "n_clicks"),
               Input("rerun-add-samples", "n_clicks"),
               Input("rerun-add-failed", "n_clicks")], 
              [State("pipeline-table", "columns"),
               State("pipeline-rerun", "derived_viewport_data"),
               State("rerun-components", "value"),
               State("rerun-samples", "value")])
def update_rerun_table_f(active, table_data, n_click_comp, n_click_samp,
                       n_click_fail, columns, prev_data, rerun_comp,
                       rerun_samp):
    return update_rerun_table(active, table_data, n_click_comp, n_click_samp,
                              n_click_fail, columns, prev_data, rerun_comp,
                              rerun_samp)


@app.callback(
    [Output("rerun-output", "children"),
     Output("rerun-output", "is_open")],
    [Input("rerun-button", "n_clicks")],
    [State("pipeline-rerun", "derived_viewport_data")],
    prevent_initial_call=True
)
def rerun_components_button_f(n_clicks, data):
    return rerun_components_button(n_clicks, data, config["rerun"])


@app.callback(Output('qc-confirm', 'displayed'),
              [Input('feedback-button', 'n_clicks_timestamp')],
              prevent_initial_call=True)
def display_confirm_feedback(button):
    if button is not None:
        return True
    return False


@app.callback(
    Output("qc-feedback", "children"),
    [Input("qc-confirm", "submit_n_clicks")],
    [State("qc-user-1", "value")] + [State("sample-radio-{}".format(n), "value")
                                     for n in range(SAMPLE_PAGESIZE)] +
                                    [State("sample_reason-{}".format(n), "value")
                                     for n in range(SAMPLE_PAGESIZE)],
    prevent_initial_call=True
)
def submit_user_feedback(_, user, *args):
    if (config["feedback_enabled"]):
        feedback_pairs = []
        for i in range(int(len(args)/2)):
            val = args[i]
            reason = args[int(len(args)/2) + i]
            if val != "noaction":
                if val.startswith("OK_"):
                    feedback_pairs.append((val[3:], "OK", reason))
                elif val.startswith("CF_"):
                    feedback_pairs.append((val[3:], "resequence", reason))
                elif val.startswith("OT_"):
                    feedback_pairs.append((val[3:], "other", reason))
        if len(feedback_pairs) > 0:
            email_config = {
                "email_from": config["email_from"],
                "email_to": config["email_to"]
            }
            import_data.add_batch_user_feedback_and_mail(feedback_pairs, user, email_config)
            return "Feedback saved"
    return []


@app.callback(
    Output("link-to-files-div", "children"),
    [Input("sample-store", "data")],
)
def link_to_files_f(data):
    return link_to_files(data)

app.clientside_callback(
    ClientsideFunction(
        namespace='clientside',
        function_name='update_selection_count'
    ),
    Output("selection-count", "children"),
    [Input("selected-samples-table", "data")],
)

app.clientside_callback(
    ClientsideFunction(
        namespace='clientside',
        function_name='enable_selection_button'
    ),
    Output("add-selection-button", "disabled"),
    [Input("datatable-ssi_stamper", "derived_viewport_selected_rows")]
)

@app.callback(
    [Output("selected-samples-table", "data"),
     Output("selection-modal-status", "children"),
     Output("selection-modal-create", "disabled")],
    [Input("add-selection-button", "n_clicks_timestamp"),
     Input("selection-modal-create", "n_clicks_timestamp")],
    [State("selected-samples-table", "data"),
     State("datatable-ssi_stamper", "derived_viewport_selected_rows"),
     State("datatable-ssi_stamper", "derived_viewport_data"),
     State("virtual_run_name", "value")
    ],
    prevent_initial_call=True
)
def selected_samples_management(add_timestamp, create_timestamp, data, selected_rows, row_data, name):
    if add_timestamp > create_timestamp:
        if (selected_rows is not None and len(selected_rows)):
            for e in selected_rows:
                row = {
                    "name": row_data[e]["name"],
                    "id": row_data[e]["id"]
                }
                if (row not in data):
                    data.append(row);
        if len(data):
            disabled = False
        else:
            disabled = True
        return [data, None, disabled]
    elif create_timestamp > add_timestamp:
        message, disabled = virtual_runs.create_run(data, name)
        return [[], message, disabled]
    else:
        if len(data):
            disabled = False
        else:
            disabled = True
        return [data, None, disabled]


# app.clientside_callback(
#     ClientsideFunction(
#         namespace='clientside',
#         function_name='open_close_selection_modal'
#     ),
#     [Output("selection-modal", "is_open"), Output("selection-modal-body", "children")],
#     [Input("selection-modal-open", "n_clicks"), Input("selection-modal-close", "n_clicks")],
#     [State("selection-modal", "is_open"),
#      State("selected-samples", "data"),
#     ]
# )
@app.callback(
    Output("selection-modal", "is_open"),
    [Input("selection-modal-open", "n_clicks"), Input("selection-modal-close", "n_clicks")],
    [State("selection-modal", "is_open")]
)
def open_close_selection_modal(n1, n2, is_open):
    if (n1 or n2):
        return not is_open
    
    return is_open

app.clientside_callback(
    ClientsideFunction(
        namespace='clientside',
        function_name='fill_full_virtual_name'
    ),
    Output("virtual_run_full_name", "children"),
    [Input("virtual_run_name", "value")],
)

server = app.server # Required for gunicorn

def main_debug():
    app.run_server(debug=True, host="0.0.0.0", dev_tools_hot_reload=True)

if __name__ == '__main__':
    # 0.0.0.0 exposes the app to the network.
    main_debug()



import dash_html_components as html
from bifrost_dashboard.components.images import list_of_images
from bifrost_dashboard.components.table import html_table, html_td_percentage
import bifrost_dashboard.components.global_vars as global_vars
import bifrost_dashboard.components.admin as admin
import dash_bootstrap_components as dbc
import pandas as pd
import numpy as np
import math

SAMPLE_PAGESIZE = 10

def get(dict_o, field, default=None):
    """
    Internal function to get an element, returning None even if it'd return a numpy None.
    """
    r = dict_o.get(field, default)
    
    if not isinstance(r, (list, np.ndarray)) and pd.isna(r):
        return default
    else:
        return r

def sample_report(data):

    sample_n = len(data)

    return [
        html.Span("0", style={"display": "none"}, id="page-n"),
        html.Span(str(sample_n // SAMPLE_PAGESIZE),
                  style={"display": "none"}, id="max-page"),
        dbc.Row(
            [
                dbc.Col(
                    [
                        html.Button(
                            "Previous page",
                            id="prevpage",
                            className="btn btn-outline-secondary",
                            n_clicks_timestamp=0),
                    ],
                    width="auto"
                ),
                dbc.Col(
                    [
                        html.Button(
                            "Next page",
                            id="nextpage",
                            className="btn btn-outline-secondary",
                            n_clicks_timestamp=0),
                    ],
                    width="auto"
                ),
            ], justify="between"
        ),

        html.Div(id="sample-report"),

        html.Div(dbc.Row(
            [
                dbc.Col(
                    [
                        html.Button(
                            "Previous page",
                            id="prevpage2",
                            className="btn btn-outline-secondary",
                            n_clicks_timestamp=0),
                    ],
                    width="auto"
                ),
                dbc.Col(
                    [
                        html.Button(
                            "Next page",
                            id="nextpage2",
                            className="btn btn-outline-secondary",
                            n_clicks_timestamp=0),
                    ],
                    width="auto"
                ),
            ], justify="between"
        ), className="mb-4"),
    ]

def check_test(test_name, sample):
    test_path = "properties.stamper.summary." + test_name + ".status"
    res = get(sample, test_path, "")
    if res == "":
        return "" # show nothing
        #return "test-missing"
    if res.startswith("pass"):
        return "test-pass"
    elif res.startswith("fail"):
        return "test-fail"
    else:
        return "test-warning"

def get_species_img(sample_data):
    genus = str(get(sample_data, "properties.species_detection.summary.name_classified_species_1")).split()[
        0].lower()
    if "{}.svg".format(genus) in list_of_images:
        img = html.Img(
            src="/assets/img/" + genus + ".svg",
            className="svg_bact"
        )
    else:
        img = None
    return img

def generate_sample_report(sample, n_sample):
    img = get_species_img(sample)
    img_div = html.Div(
            [
                img
            ],
            className="d-inline")
    return (
        dbc.Card(
            [
                dbc.CardHeader([
                    img_div,
                    html.Div([
                        html.A(id="sample-" + str(sample["name"])),
                        html.H6(
                            sample["name"],
                            className="d-inline font-weight-bold text-primary mx-2"),
                        html.I(get(sample, "properties.species_detection.summary.species"))
                    ], className="d-inline-block"),
                    
                ]),
                dbc.CardBody([
                    html_sample_tables(sample),
                    admin.sample_radio_feedback(sample, n_sample)
                ])
                
            ], className="shadow mb-4"
        )
    )


def html_organisms_table(sample_data, **kwargs):
    percentages = [
        get(sample_data, "properties.species_detection.summary.percent_classified_species_1", math.nan),
        get(sample_data, "properties.species_detection.summary.percent_classified_species_2", math.nan),
        get(sample_data, "properties.species_detection.summary.percent_unclassified", math.nan)
    ]

    color_0 = "#b3ccc1"
    color_1 = "#b3ccc1" #Green

    color_2 = "#f3bbd3"  # Default

#   color_u = COLOR_DICT.get("", "#fee1cd")  # Default
    color_u = "#fee1cd"  # Default

    return html.Div([
        html.Table([
            html.Tr([
                html.Td(
                    [html.I(get(sample_data, "properties.species_detection.summary.name_classified_species_1", "No data")), " + Unclassified"], className="cell"),
                html_td_percentage(percentages[0] + percentages[2], color_0)
            ], className=check_test("test__species_detection__main_species_level", sample_data) + " trow"),
            html.Tr([
                html.Td(
                    html.I(get(sample_data, "properties.species_detection.summary.name_classified_species_1", "No data")), className="cell"),
                html_td_percentage(percentages[0], color_1)
            ], className="trow"),
            html.Tr([
                html.Td(
                    html.I(get(sample_data, "properties.species_detection.summary.name_classified_species_2", "No data")), className="cell"),
                html_td_percentage(percentages[1], color_2)
            ], className="trow"),
            html.Tr([
                html.Td("Unclassified", className="cell"),
                html_td_percentage(percentages[2], color_u)
            ], className=check_test("test__species_detection__unclassified_level", sample_data) + " trow")
        ], className="bifrost-table")
    ], **kwargs)

def html_test_tables(sample_data):
    qc_to_check = ["stamper", "user_feedback"]
    rows = []
    for key, value in sample_data.items():
        if key.startswith("properties.stamper.summary.whats_my_species") \
                or key.startswith("properties.stamper.summary.assemblatron"):
            if pd.isnull(value):
                value = "nan"
            name_v = key.split(":")
            values = value.split(':')
            if len(values) == 3:
                if values[0] != "pass":
                    rows.append([name_v[1].capitalize(), values[1]])
            if (key.endswith(".action")):
                rows.append(["QC Action", value])

    qc_rows = []
    for qc_cat in qc_to_check:
        qc_key = "properties.{}.summary.stamp".format(qc_cat)
        qc_key_val = "{}.value".format(qc_key)
        qc_val = get(sample_data, qc_key_val)
        if qc_val is not None:
            if str(qc_val).startswith("OK"):
                qc_class = "test-pass"
            elif str(qc_val) in ["core facility", "supplying lab", "other"]:
                qc_class = "test-fail"
            else:
                qc_class = ""
            source = get(sample_data, "{}.display_name".format(qc_key))
            reason = get(sample_data, "{}.reason".format(qc_key), "")
            qc_label = "{} [{}, {}]".format(qc_cat, source, reason)
            qc_rows.append({
                "list": [qc_label, qc_val],
                "className": qc_class
            })

    if len(rows):
        test_table = html_table(rows)
    else:
        test_table = html.P("No failed tests.")
    
    if len(qc_rows):
        qc_table = html_table(qc_rows)
    else:
        qc_table = html.P("No info.")

    return dbc.Row([
        dbc.Col([
            html.H6("QC", className="table-header"),
            qc_table
        ]),
        dbc.Col([
            html.H6("Failed QC tests", className="table-header"),
            test_table
        ]),
    ])


def html_sample_tables(sample_data):
    """Generate the tables for each sample containing submitter information,
       detected organisms etc. """

    if get(sample_data, "properties.sample_info.summary.sample_name"):
        emails_l = get(sample_data, "properties.sample_info.summary.emails")
        if emails_l:
            n_emails = len(emails_l.split(";"))
            if (n_emails > 1):
                emails = ", ".join(emails_l.split(";")[:2])
                if (n_emails > 2):
                    emails += ", ..."
            else:
                emails = emails_l
        else:
            emails = ""
        sample_sheet_table = html_table([
                    ["Supplied name", get(sample_data, "sample_sheet.sample_name", "")],
                    ["User Comments", get(sample_data, "sample_sheet.Comments", "")],
                    ["Supplying lab", get(sample_data, "sample_sheet.group", "")],
                    ["Submitter emails", emails],
                    {
                        "list": ["Provided species", html.I(
                            get(sample_data, "sample_sheet.provided_species", ""))],
                        "className": check_test("test__sample__species_provided_is_detected", sample_data)
                    },
                    {
                        "list": ["Read file", 
                                 str(get(sample_data, "properties.datafiles.summary.paired_reads", [""])[0]).split("/")[-1]],
                        "className": check_test("test__sample__has_reads_files", sample_data)
                    }
                ])
    else:
        sample_sheet_table = html_table([])

    title = "Assemblatron Results"
    table = html.Div([
        html_table([
            {
                "list": [
                    "Number of filtered reads",
                    "{:,.0f}".format(
                        get(sample_data, "properties.denovo_assembly.summary.filtered_reads_num", math.nan))
                ],
                "className": check_test("test__denovo_assembly__minimum_read_number", sample_data)
            },
            [
                "Number of contigs (1x cov.)",
                "{:,.0f}".format(
                    get(sample_data, "properties.denovo_assembly.summary.bin_contigs_at_1x", math.nan))
            ],
            [
                "Number of contigs (10x cov.)",
                "{:,.0f}".format(
                    get(sample_data, "properties.denovo_assembly.summary.bin_contigs_at_10x", math.nan))
            ],
            [
                "N50",
                "{:,}".format(
                    get(sample_data, "properties.denovo_assembly.summary.N50", math.nan))
            ],
            {
                "list": [
                    "Average coverage (1x)",
                    "{:,.2f}".format(
                        get(sample_data, "properties.denovo_assembly.summary.bin_coverage_at_1x", math.nan))
                ],
                "className": check_test("test__denovo_assembly__genome_average_coverage", sample_data)
            },
            {
                "list": [
                    "Genome size at 1x depth",
                    "{:,.0f}".format(
                        get(sample_data, "properties.denovo_assembly.summary.bin_length_at_1x", math.nan))
                ],
                "className": check_test("test__denovo_assembly__genome_size_at_1x", sample_data)
            },
            {
                "list": [
                    "Genome size at 10x depth",
                    "{:,.0f}".format(
                        get(sample_data, "properties.denovo_assembly.summary.bin_length_at_10x", math.nan))
                ],
                "className": check_test("test__denovo_assembly__genome_size_at_10x", sample_data)
            },
            {
                "list": [
                    "Genome size 1x - 10x diff",
                    "{:,.0f}".format(
                        get(sample_data, "properties.denovo_assembly.summary.bin_length_at_1x", math.nan)
                        - get(sample_data, "properties.denovo_assembly.summary.bin_length_at_10x", math.nan)
                    )
                ],
                "className": check_test("test__denovo_assembly__genome_size_difference_1x_10x", sample_data)
            },
            [
                "Genome size at 25x depth",
                "{:,.0f}".format(
                    get(sample_data, "properties.denovo_assembly.summary.bin_length_at_25x", math.nan))
            ],
            [
                "Ambiguous sites",
                "{:,.0f}".format(
                    get(sample_data, "properties.denovo_assembly.summary.snp_filter_10x_10%", math.nan))
            ] 
        ])
    ])
    expected_results = global_vars.expected_results
    any_results = False
    results = []
    for entry in expected_results:
        entry_key = "report.{}.data".format(entry)
        if entry_key in sample_data:
            any_results = True
            if len(get(sample_data, entry_key, [])) == 0:
                datatable = "No results"
            else:
                data = get(sample_data, "report.{}.data".format(entry), [])
                columns = get(sample_data, "report.{}.columns".format(entry), [])
                header = [c["name"] for c in columns]
                rows = []
                for rowd in data:

                    row = [rowd[col["id"]] for col in columns]
                    rows.append(row)

                datatable = html_table(rows, header)
                # dt.DataTable(
                #     style_table={
                #         'overflowX': 'scroll',
                #         'overflowY': 'scroll',
                #         'maxHeight': '480'
                #     },

                #     columns=get(
                #         sample_data, "report.{}.columns".format(entry), []),
                #     data=get(sample_data, "report.{}.data".format(entry), []),
                #     page_action='none'
                # )
            results.append(html.Div([
                html.H6(get(sample_data, "report.{}.title".format(entry)),
                        className="table-header"),
                html.Div(get(sample_data, "report.{}.info".format(entry))),
                html.Div(
                    datatable,
                    className="grey-border")
            ], className="col-6"))
        else:
            results.append(html.Div([
                html.H6("{} not run".format(entry.capitalize()),
                        className="table-header")
            ], className="col-6"))

    # mlst_db = sample_data.get("ariba_mlst.mlst_db", "")

    if any_results:
        res_div = html.Details([
            html.Summary(
                "ResFinder/PlasmidFinder/VirulenceFinder/MLST (click to show)"),
            html.Div([
                html.Div(results, className="row"),
            ])
        ])
    else:
        res_div = html.Div(
            html.P("Resfinder, plasmidfinder, MLST and virulencefinder were not run."))


    mlst_type = "ND"
    strain_list = get(sample_data, "properties.mlst.summary.strain")
    if isinstance(strain_list, (list, np.ndarray)):
        mlst_type = ", ".join(list(map(str, strain_list)))


    return html.Div([
        dbc.Row([
            dbc.Col([
                html.H6("Sample Sheet", className="table-header"),
                sample_sheet_table,
                html.H6("Detected Organisms", className="table-header"),
                html_organisms_table(sample_data)
            ]),
            dbc.Col([
                html.H6(title, className="table-header"),
                table,
                html.H6("MLST type: {}".format(mlst_type), className="table-header"),
            ])
        ]),
        html_test_tables(sample_data),
        res_div
    ])


def children_sample_list_report(dataframe):
    report = []
    row_index = 0
    for _, sample in \
            dataframe.iterrows():
        report.append(generate_sample_report(sample,
                                             row_index))
        row_index += 1
    return html.Div(report)

# callback
def samples_next_page(prev_ts, prev_ts2, next_ts, next_ts2, page_n, max_page):
    page_n = int(page_n)
    max_page = int(max_page)
    if max(prev_ts, prev_ts2) > max(next_ts, next_ts2):
        return str(max(page_n - 1, 0))
    elif max(next_ts, next_ts2) > max(prev_ts, prev_ts2):
        return str(min(page_n + 1, max_page))
    else:
        return '0'



import re
import datetime

import dash_bootstrap_components as dbc
import dash_html_components as html
import dash_table
from flask import request
import bifrostapi

modal = dbc.Modal(
    [
        dbc.ModalHeader("Selected samples"),
        dbc.ModalBody([
            dash_table.DataTable(id="selected-samples-table",
                                data=[],
                                columns=[{"id": "id", "name": "id"},{"id": "name", "name": "name"}],
                                row_deletable=True),
            html.Hr(),
            dbc.Form([
                dbc.FormGroup([
                    dbc.Label("Virtual run name ", html_for="virtual_run_name"),
                    dbc.Input(type="text", id="virtual_run_name", pattern="^[a-zA-Z0-9_-]+$"),
                    dbc.FormText(["Only accepts 'A-Za-z0-9-_'"] ,color="secondary"),
                    dbc.FormText(["Full name will be: ", html.Span(id="virtual_run_full_name")] ,color="secondary")
                ])
            ]),
            html.Div([], id="selection-modal-status")
        ]),
        dbc.ModalFooter([
            dbc.Button("Close", id="selection-modal-close", className="ml-auto"),
            dbc.Button("Create virtual run", id="selection-modal-create", 
                       color="primary", n_clicks_timestamp=-1, disabled=True)
        ]),
    ],
    id="selection-modal",
)

def create_run(samples, name):
    if len(name) == 0:
        return "You must enter a run name", False
    name = name.upper()
    if re.match("^[a-zA-Z0-9_-]+$", name) is None:
        return "Invalid name. Name should only contain alphanumeric characters, - or _", False
    if len(samples) == 0:
        return "Run must contain at least one sample", False

    day = datetime.date.today().strftime("%y%m%d")
    name = f"{day}_CUSTOM_SSIVRT_{name}"

    ip = request.remote_addr
    try:
        bifrostapi.create_virtual_run(name, ip, samples)
    except ValueError:
        return "Run could not be created, check that the name is not taken.", False

    return [["Run created successfully. You can access it ", html.A("here", href="/collection/" + name)], True]